import { generate as bfgen } from "./lib/brainfuck.js";

const vm = require("vm")
  , request = require("request");

const maxoutput = 400;

module.exports = bot => {
  bot._trigger.set("sandbox_js", {
    call: /^\!js (.*)/i,
    level: 100,
    active: true,
    clients: ["irc", "tg"],
    f: e => {
      const args = e.message.substring(3);
      const context = {
        e: e,
        t: bot._trigger
      };
      try {
        let output = vm.runInNewContext(args, context);
        if (typeof output !== undefined && output) {
          output = JSON.stringify(output);
          if (output !== "Converting circular structure to JSON") {
            e.reply(output.length > maxoutput ? `holy fuck, Ausgabe wäre viel zu lang! (${output.length} Zeichen :DDDDDD)` : output);
          }
        }
      }
      catch (err) {
        e.reply(err.message);
      }
    }
  });

  function sandbox(lang, code) {
    const langs = {
      cpp: {
        "LanguageChoice": "7",
        "Program": "#include <iostream>\r\nint main() { " + code + "}",
        "Input": "",
        "CompilerArgs": "-Wall -std=c++14 -O2 -o a.out source_file.cpp"
      },
      hs: {
        "LanguageChoice": "11",
        "Program": "main = " + code,
        "Input": "",
        "CompilerArgs": "-o a.out source_file.hs"
      },
      py: {
        "LanguageChoice": "24",
        "Program": "print(" + code + ")",
        "Input": "",
        "CompilerArgs": "-o a.out source_file.hs"
      },
      bf: {
        "LanguageChoice": "44",
        "Program": code,
        "Input": "",
        "CompilerArgs": ""
      },
      php: {
        "LanguageChoice": "8",
        "Program": "<?php\r\n" + code + "\r\n?>",
        "Input": "",
        "CompilerArgs": ""
      },
      lua: {
        "LanguageChoice": "14",
        "Program": code,
        "Input": "",
        "CompilerArgs": ""
      }
    };

    return new Promise((resolve, reject) => {
      const options = {
        url: "http://rextester.com/rundotnet/api",
        method: "POST",
        body: langs[lang],
        json: true
      };

      request(options, (err, res, body) => {
        if(err)
          return reject("Error!");
        if(body.Errors) {
          if(body.Errors.length > maxoutput)
            return reject(`holy fuck, Error wäre viel zu lang! (${body.Errors.length} Zeichen :DDDDDD)`)
          return reject(body.Errors);
        }
        if(body.Result === null)
          return reject("Keine Ausgabe :(");
        if(body.Result.length > maxoutput)
          return reject(`holy fuck, Ausgabe wäre viel zu lang! (${body.Result.length} Zeichen :DDDDDD)`);
        resolve(body.Result);
      });
    });
  }

  bot._trigger.set("sandbox", {
    call: /^\!(hs|py|cpp|bf|php|lua) .*/i,
    level: 100,
    active: true,
    clients: ["irc", "tg"],
    f: e => {
      let args = e.message.trim().substring(1).split(" ");
      const lang = args.shift();
      args = args.join(" ");

      sandbox(lang, args).then(
        resolve => e.reply(resolve),
        reject => e.reply(reject.replace("\n", " "))
      );
    }
  });

  bot._trigger.set("sandbox_rs", {
    call: /^\!rs (.*)/i,
    level: 100,
    active: true,
    clients: ["irc", "tg"],
    f: e => {
      const args = e.message.substring(4);
      const params = {
        channel: "stable",
        code: "fn main() { " + args + " }",
        crateType: "bin",
        mode: "debug",
        tests: false
      };
      const options = {
        url: "https://play.rust-lang.org/execute",
        method: "POST",
        body: params,
        json: true
      };

      request(options, (err, res, body) => {
        e.reply(body.success ? (body.stdout.length > maxoutput ? `holy fuck, Ausgabe wäre viel zu lang! (${body.stdout.length} Zeichen :DDDDDD)` : body.stdout) : "error!");
      });
    }
  });

  bot._trigger.set("bfgen", {
    call: /^\!bfgen .*/i,
    level: 100,
    active: true,
    clients: ["irc", "tg"],
    f: e => {
      let args = e.message.trim().substring(7);
      e.reply(bfgen(args));
    }
  });
};
